from pif.utils import get_public_ip

__title__ = 'pif'
__version__ = '0.7.4'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('get_public_ip',)



import logging

from requests import get

from pif.base import BasePublicIPChecker, registry

__title__ = 'pif.checkers.icanhazip.com.pif_ip_checker'
__author__ = 'Bruno Santeramo'
__copyright__ = 'Copyright (c) 2016 Bruno Santeramo'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('IcanhazipIPChecker',)

logger = logging.getLogger(__name__)


class IcanhazipIPChecker(BasePublicIPChecker):
    """Check IPs using icanhazip.com."""

    uid = 'icanhazip.com'

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        try:
            data = get('http://icanhazip.com/', verify=False).text.rstrip()
            return data
        except Exception as err:
            if self.verbose:
                logger.debug(err)


registry.register(IcanhazipIPChecker)



from . import defaults

__title__ = 'pif.conf'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_setting',
    'set_setting',
    'settings',
    'reset_to_defaults_settings',
)


class Settings(object):
    """Settings registry."""

    def __init__(self):
        self._settings = {}

    def set(self, name, value):
        """Override default settings.

        :param str name:
        :param mixed value:
        """
        self._settings[name] = value

    def get(self, name, default=None):
        """Get a variable from local settings.

        :param str name:
        :param mixed default: Default value.
        :return mixed:
        """
        if name in self._settings:
            return self._settings.get(name, default)
        elif hasattr(defaults, name):
            return getattr(defaults, name, default)
        else:
            return default

    def reset_to_defaults(self):
        """Resets settings to defaults."""
        self._settings = {}


settings = Settings()

get_setting = settings.get

set_setting = settings.set

reset_to_defaults_settings = settings.reset_to_defaults



__title__ = 'pif.benchmark'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('run_benchmark_test',)


def run_benchmark_test():
    """Run the benchmark test to identify the fastest provider."""



import os

__title__ = 'pif.helpers'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'project_dir',
    'PROJECT_DIR',
)


def project_dir(base):
    return os.path.abspath(
        os.path.join(
            os.path.dirname(__file__),
            (os.path.join(*base) if isinstance(base, (list, tuple)) else base)
        ).replace('\\', '/')
    )


PROJECT_DIR = project_dir



import socket

from .conf import get_setting
from .exceptions import InvalidRegistryItemType

__title__ = 'pif.base'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BasePublicIPChecker', 'PublicIPCheckerRegistry', 'registry')


class BasePublicIPChecker(object):
    """Base public IP checker."""

    uid = None
    verbose = False

    def __init__(self, verbose=False):
        """Constructor.

        :param bool verbose:
        """
        assert self.uid
        self.verbose = verbose

    def get_local_ip(self):
        """Get local IP.

        :return str:
        """
        return socket.gethostbyname(socket.gethostname())

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        raise NotImplemented("You should override ``get_ip`` method in your "
                             "IP checker class.")


class PublicIPCheckerRegistry(object):
    """Registry of public IP checkers."""

    def __init__(self):
        self._registry = {}
        self._forced = []

    def register(self, cls):
        """Register the IP checker in the registry.

        :param pif.base.BaseIPChecker cls: Subclass of
            ``pif.base.BaseIPChecker``.
        :param bool force: If set to True, item stays forced. It's not
            possible to un-register a forced item.
        :return bool: True if registered and False otherwise.
        """
        if not issubclass(cls, BasePublicIPChecker):
            raise InvalidRegistryItemType(
                "Invalid item type `%s` for registry "
                "`%s`" % (cls, self.__class__)
            )

        if cls.uid not in self._registry:
            self._registry[cls.uid] = cls
            return True
        else:
            return False

    def unregister(self, checker):
        """Un-registers an item from registry.

        :param mixed checker: May be a subclass of
            ``pif.base.BasePublicIPChecker`` or string, representing the
            checker name.
        :return bool: True if unregistered and False otherwise.
        """
        # If not string, then we check it it's of a right class.
        if not isinstance(checker, str):
            if not issubclass(checker, BasePublicIPChecker):
                raise InvalidRegistryItemType(
                    "Invalid item type `%s` for registry "
                    "`%s`" % (checker, self.__class__)
                )
            # So, it's the right class.

            checker = checker.uid

        if checker in self._registry:
            self._registry.pop(checker)
            return True
        else:
            return False

    def get(self, uid):
        return self._registry.get(uid, None)


# Register public IP checkers by calling registry.register()
registry = PublicIPCheckerRegistry()



class InvalidRegistryItemType(ValueError):
    """Raised when an attempt is made to register an item in the registry
    which does not have a proper type.
    """


__title__ = 'pif.exceptions'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('InvalidRegistryItemType',)



from __future__ import print_function

import logging
import unittest

from pif.discover import autodiscover
from pif.utils import get_public_ip, list_checkers
from pif.base import registry, BasePublicIPChecker

__title__ = 'pif.tests'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('log_info', 'PifTest',)

LOG_INFO = True
TRACK_TIME = False

logger = logging.getLogger(__name__)


def log_info(func):
    """Prints some useful info."""
    if not LOG_INFO:
        return func

    def inner(self, *args, **kwargs):
        if TRACK_TIME:
            import simple_timer
            timer = simple_timer.Timer()  # Start timer

        result = func(self, *args, **kwargs)

        if TRACK_TIME:
            timer.stop()  # Stop timer

        logger.info('\n{0}'.format(func.__name__))
        logger.info('============================')
        logger.info('""" {0} """'.format(func.__doc__.strip()))
        logger.info('----------------------------')
        if result is not None:
            logger.info(result)
        if TRACK_TIME:
            logger.info('done in {0} seconds'.format(timer.duration))

        return result
    return inner


class PifTest(unittest.TestCase):
    """Tests."""

    def setUp(self):
        """Set up."""
        pass

    @log_info
    def test_01_autodiscover(self):
        """Test ``autodiscover``."""
        autodiscover()
        self.assertTrue(len(registry._registry) > 0)

    @log_info
    def test_02_get_public_ip(self):
        """Test get IP."""
        res = get_public_ip(verbose=True)
        self.assertIsNotNone(res)
        return res

    @log_info
    def _test_get_public_ip_using_preferred_checker(self, checker):
        """Test get_public_ip using preferred checker."""
        res = get_public_ip(checker, verbose=True)
        return res

    @log_info
    def test_03_get_public_ip_using_preferred_checker(self):
        """Test get IP using preferred checker `ident.me`."""
        checkers = list_checkers()
        res = {}
        failed = []
        for checker in checkers:
            _res = self._test_get_public_ip_using_preferred_checker(checker)
            if _res is None:
                failed.append(checker)
            res.update({checker: _res})
        self.assertTrue(
            len(failed) == 0,
            "Failed for {}".format(','.join(failed))
        )
        return res

    # @log_info
    # def test_04_get_public_ip_using_preferred_checker_ident(self):
    #     """Test get IP using preferred checker `ident.me`."""
    #     res = get_public_ip('ident.me', verbose=True)
    #     self.assertIsNotNone(res)
    #     return res
    #
    # @log_info
    # def test_05_get_public_ip_using_preferred_checker_dyndns(self):
    #     """Test get IP using preferred checker `dyndns.com`."""
    #     res = get_public_ip('dyndns.com', verbose=True)
    #     self.assertIsNotNone(res)
    #     return res

    @log_info
    def test_04_list_checkers(self):
        """Lists all registered checkers."""
        res = list_checkers()
        self.assertTrue(len(res) > 0)
        return res

    @log_info
    def test_05_unregister_checker(self):
        """Test un-register checker `dyndns.com`."""
        self.assertTrue('dyndns.com' in registry._registry.keys())
        registry.unregister('dyndns.com')
        self.assertTrue('dyndns.com' not in registry._registry.keys())

    @log_info
    def test_06_register_custom_checker(self):
        """Test un-register checker `dyndns`."""
        class MyPublicIPChecker(BasePublicIPChecker):
            uid = 'mypublicipchecker'

            def get_public_ip(self):
                return '8.8.8.8'

        registry.register(MyPublicIPChecker)
        self.assertTrue('mypublicipchecker' in registry._registry.keys())


if __name__ == '__main__':
    unittest.main()



import logging

from requests import get

from json import loads

from pif.base import BasePublicIPChecker, registry

__title__ = 'pif.checkers.httpbin.org.pif_ip_checker'
__author__ = 'Bruno Santeramo'
__copyright__ = 'Copyright (c) 2016 Bruno Santeramo'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('HttpbinIPChecker',)


logger = logging.getLogger(__name__)


class HttpbinIPChecker(BasePublicIPChecker):
    """Checks IPs using httpbin.org."""

    uid = 'httpbin.org'

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        try:
            data = loads(get('http://httpbin.org/ip', verify=False).text)
            return data['origin']
        except Exception as err:
            if self.verbose:
                logger.debug(err)


registry.register(HttpbinIPChecker)



import logging

from requests import get

from pif.base import BasePublicIPChecker, registry

__title__ = 'pif.checkers.ident.pif_ip_checker'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'IdentmeIPChecker',
    'V4IdentmeIPChecker',
    'V6IdentmeIPChecker'
)

logger = logging.getLogger(__name__)


class IdentmeIPChecker(BasePublicIPChecker):
    """Check IPs using ident.me."""

    uid = 'ident.me'

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        try:
            data = get('http://ident.me', verify=False).text
            return data
        except Exception as err:
            if self.verbose:
                logger.error(err)


registry.register(IdentmeIPChecker)

# ********************************************************
# ********************************************************
# ********************************************************


class V4IdentmeIPChecker(BasePublicIPChecker):
    """Checks IPs using v4.ident.me."""

    uid = 'v4.ident.me'

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        try:
            data = get('http://v4.ident.me', verify=False).text
            return data
        except Exception as err:
            if self.verbose:
                logger.error(err)


registry.register(V4IdentmeIPChecker)

# ********************************************************
# ********************************************************
# ********************************************************


class V6IdentmeIPChecker(BasePublicIPChecker):
    """Check IPs using v6.ident.me."""

    uid = 'v6.ident.me'

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        try:
            data = get('http://v6.ident.me', verify=False).text
            return data
        except Exception as err:
            if self.verbose:
                logger.error(err)


registry.register(V6IdentmeIPChecker)



from __future__ import print_function

import logging

from pif.base import registry
from pif.discover import autodiscover

__title__ = 'pif.utils'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('ensure_autodiscover', 'list_checkers', 'get_public_ip',)

logger = logging.getLogger(__name__)


def ensure_autodiscover():
    """Ensure the IP checkers are auto-discovered."""
    if not registry._registry:
        autodiscover()


def list_checkers():
    """List available checkers.

    :return list:
    """
    ensure_autodiscover()
    return registry._registry.keys()


def get_public_ip(preferred_checker=None, verbose=False):
    """Get IP using one of the services.

    :param str preffered checker: Checker UID. If given, the preferred
        checker is used.
    :param bool verbose: If set to True, debug info is printed.
    :return str:
    """
    ensure_autodiscover()

    # If use preferred checker.
    if preferred_checker:
        ip_checker_cls = registry.get(preferred_checker)

        if not ip_checker_cls:
            return False

        ip_checker = ip_checker_cls(verbose=verbose)
        ip = ip_checker.get_public_ip()

        if verbose:
            # logger.info('provider: ', ip_checker_cls)
            logger.info('provider: ', preferred_checker)
        return ip

    # Using all checkers.

    for ip_checker_name, ip_checker_cls in registry._registry.items():
        ip_checker = ip_checker_cls(verbose=verbose)
        try:
            ip = ip_checker.get_public_ip()
            if ip:
                if verbose:
                    logger.info('provider: ', ip_checker_cls)
                return ip

        except Exception as e:
            if verbose:
                print(e)

    return False



__title__ = 'pif.defaults'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('IP_CHECKERS_DIR', 'IP_CHECKER_MODULE_NAME', 'DEBUG',)

IP_CHECKERS_DIR = 'checkers'
IP_CHECKER_MODULE_NAME = 'pif_ip_checker'

DEBUG = False



from __future__ import print_function

import os
import logging

from .helpers import PROJECT_DIR
from .conf import get_setting

try:
    from importlib import import_module
except ImportError:
    import_module = __import__

__title__ = 'pif.discover'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('autodiscover',)


logger = logging.getLogger(__name__)


def autodiscover():
    """Autodiscovers the pif IP checkers in checkers directory."""
    IP_CHECKERS_DIR = get_setting('IP_CHECKERS_DIR')
    IP_CHECKER_MODULE_NAME = get_setting('IP_CHECKER_MODULE_NAME')
    DEBUG = get_setting('DEBUG')

    for app_path in os.listdir(PROJECT_DIR(IP_CHECKERS_DIR)):
        full_app_path = [IP_CHECKERS_DIR]
        full_app_path.append(app_path)
        if os.path.isdir(PROJECT_DIR(full_app_path)):
            try:
                import_module(
                    "pif.%s.%s.%s" % (IP_CHECKERS_DIR,
                                      app_path,
                                      IP_CHECKER_MODULE_NAME)
                )
            except ImportError as e:
                if DEBUG:
                    logger.debug(str(e))
            except Exception as e:
                if DEBUG:
                    logger.debug(str(e))
        else:
            pass



import logging
import re

from requests import get

from pif.base import BasePublicIPChecker, registry

__title__ = 'pif.checkers.whatismyip.pif_ip_checker'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('WhatismyipIPChecker',)

logger = logging.getLogger(__name__)


class WhatismyipIPChecker(BasePublicIPChecker):
    """Checks IPs using whatismyip.com."""

    uid = 'whatismyip.com'

    def get_public_ip(self):
        """Gets public IP.

        :return str:
        """
        try:
            data = get(
                'http://www.whatismyip.com/ip-address-lookup/',
                verify=False
            ).text
            return re.compile(r'name="ip"(.*) value="(.+)"') \
                     .search(data) \
                     .group(2)
        except Exception as err:
            if self.verbose:
                logger.error(err)


# registry.register(WhatismyipIPChecker)



import logging
import re

from requests import get

from pif.base import BasePublicIPChecker, registry

__title__ = 'pif.checkers.dyndns.pif_ip_checker'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('DyndnsIPChecker',)

logger = logging.getLogger(__name__)


class DyndnsIPChecker(BasePublicIPChecker):
    """Checks IPs using checkip.dyndns.com."""

    uid = 'dyndns.com'

    def get_public_ip(self):
        """Get public IP.

        :return str:
        """
        try:
            data = get('http://checkip.dyndns.com/', verify=False).text
            return re.compile(r'Address: (\d+\.\d+\.\d+\.\d+)') \
                     .search(data) \
                     .group(1)
        except Exception as err:
            if self.verbose:
                logger.debug(err)


registry.register(DyndnsIPChecker)



import os
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

version = '0.7.4'

exec_dirs = [
    'src/pif/bin/',
]

execs = []
for exec_dir in exec_dirs:
    execs += [os.path.join(exec_dir, f) for f in os.listdir(exec_dir)]

setup(
    name='pif',
    version=version,
    description="Public IP address checker.",
    long_description=readme,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Topic :: Internet"
    ],
    keywords='public ip checker, ip',
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    url='https://github.com/barseghyanartur/pif',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    include_package_data=True,
    package_data={
        'pif': execs,
    },
    scripts=['src/pif/bin/get-public-ip',],
    license='GPL 2.0/LGPL 2.1',
    install_requires=['requests>=1.2.3', 'argparse']
)



from __future__ import print_function

import argparse

from pif.utils import get_public_ip

__title__ = 'pif.commands.get_public_ip'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('main',)


def main():
    """Get public IP.

    :example:

    $ python src/pif/get_public_ip.py -pc user
    """
    parser = argparse.ArgumentParser(
        description="""Get public IP."""
    )

    parser.add_argument("-c", "--checker",
                        dest="preferred_checker",
                        type=str,
                        help="`preferred_checker` value",
                        metavar="PREFERRED_CHECKER")
    parser.add_argument("-v", "--verbose",
                        dest="verbose",
                        type=str,
                        help="`verbose` value",
                        metavar="VERBOSE")
    args = parser.parse_args()

    kwargs = {}

    if args.preferred_checker:
        kwargs.update({'preferred_checker': args.preferred_checker})

    verbose = None

    try:
        verbose = bool(int(args.verbose))
        kwargs.update({'verbose': verbose})
    except Exception as err:
        pass

    try:
        public_ip = get_public_ip(**kwargs)
        print(public_ip)
    except Exception as e:
        print(e)


if __name__ == "__main__":
    main()
